/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*
 * ======== Master_HAL_FR5969.c ========
 * Modules and GPIOs used by this HAL:
 *      =================================
 *      |   I2C         |   eUSCIB0     |
 *      |   I2C_SDA     |    P1.6       |
 *      |   I2C_SCL     |    P1.7       |
 *      |   UART        |   eUSCIA0     |
 *      |   UART_RXD    |    P2.1       |
 *      |   UART_TXD    |    P2.0       |
 *      |   LED0        |    P1.0       |
 *      |   LED1        |    P4.6       |
 *      |   TMP006_DRDY |    P3.5       |
 *      =================================
 * ---------------------------------------------------------------------------*/

#include <stdint.h>
#include "driverlib.h"
#include "Master_HAL.h"
#include "smbus.h"

// 
// Function prototypes
//
static void MCU_Clock_Init(void);
static void MCU_GPIO_Init(void);


//*****************************************************************************
//
//! MCU initialization 
//!
//! Initializes the MSP430
//!  Sets GPIOs to default state
//!  Configures clocks
//!
//! \return None
//
// *****************************************************************************
void HAL_MCU_Init(void)
{

    // Initialize GPIOs 
    MCU_GPIO_Init();

    // Initialize the MSP430 Clocks
    MCU_Clock_Init();
    
    // Turn off Temp sensor
    REFCTL0 |= REFTCOFF; 
    REFCTL0 &= ~REFON;   
    
    return;
}

//*****************************************************************************
//
//! Clock Initialization 
//!
//! This demo sets SMCLK/MCLK = DCO @ MCLK_MHZ (defined in project settings)
//!                ACLK = VLO
//!
//! \return None
//
// *****************************************************************************
static void MCU_Clock_Init(void)
{

#if (MCLK_MHZ == 1)
    //Set DCO Frequency to 8MHz
    CS_setDCOFreq(CS_DCORSEL_0,CS_DCOFSEL_6);
    //Initialize SMCLK=DCOCLK and CLK DIVIDER set to 8 = 1Mhz
    CS_clockSignalInit(CS_SMCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_8);
    //Initialize MCLK=DCOCLK and CLK DIVIDER set to 8 = 1Mhz
    CS_clockSignalInit(CS_MCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_8);
#elif (MCLK_MHZ == 8)
    //Set DCO Frequency to 8MHz
    CS_setDCOFreq(CS_DCORSEL_0,CS_DCOFSEL_6);
    //Initialize SMCLK=DCOCLK and CLK DIVIDER set to 1 = 8Mhz
    CS_clockSignalInit(CS_SMCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
    //Initialize MCLK=DCOCLK and CLK DIVIDER set to 1 = 8Mhz
    CS_clockSignalInit(CS_MCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
#elif (MCLK_MHZ == 16)
    //Set DCO Frequency to 16MHz
    CS_setDCOFreq(CS_DCORSEL_1,CS_DCOFSEL_4);
    //Initialize SMCLK=DCOCLK and CLK DIVIDER set to 1 = 16Mhz
    CS_clockSignalInit(CS_SMCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
    //Initialize MCLK=DCOCLK and CLK DIVIDER set to 1 = 16Mhz
    CS_clockSignalInit(CS_MCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
#elif (MCLK_MHZ == 24)
    //Set DCO Frequency to 24MHz
    CS_setDCOFreq(CS_DCORSEL_1,CS_DCOFSEL_6);
    //Initialize SMCLK=DCOCLK and CLK DIVIDER set to 1 = 24Mhz
    CS_clockSignalInit(CS_SMCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
    //Initialize MCLK=DCOCLK and CLK DIVIDER set to 1 = 24Mhz
    CS_clockSignalInit(CS_MCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
#else 
#error "Frequency not supported, change project setttings"
#endif
    
    //Initialize ACLK=VLO and no CLK DIVIDER set to 1
    CS_clockSignalInit(CS_ACLK,CS_VLOCLK_SELECT,CS_CLOCK_DIVIDER_1);

    // Crystals are not used
    CS_HFXTOff();
    CS_LFXTOff();
    
    return;
}


//*****************************************************************************
//
//! GPIO Initialization 
//!
//! Set GPIO pins to a default state
//!
//! \return None
//
// *****************************************************************************
static void MCU_GPIO_Init(void)
{
    // Set all pins as output low, except 
    // I2C (P1[6:7]) 
    // UART (P2[0:1])
    // P1.4 which is used by slave to drive LED1 in TMP006EVM when 
    //  stacked on top of MSP-EXP430FR5969
    // P4.2 which can be connected to LED0 from slave
    GPIO_setOutputLowOnPin(GPIO_PORT_P1, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|
                                        GPIO_PIN5);
    GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|
                                        GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P3, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|
                                        GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|
                                          GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN0|GPIO_PIN1|
                                        GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|
                                          GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_PJ, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|
                                        GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|
                                          GPIO_PIN6|GPIO_PIN7);

    GPIO_setAsOutputPin(GPIO_PORT_P1, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|
                                        GPIO_PIN5);
    GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|
                                    GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P3, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|
                                    GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P4, GPIO_PIN0|GPIO_PIN1|GPIO_PIN3|
                                    GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_PJ, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|
                                    GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);


    // Disable the GPIO power-on default high-impedance mode to activate
	// previously configured port settings
	PM5CTL0 &= ~LOCKLPM5;

    return;
}

//*****************************************************************************
//
//! Set LED high
//!
//! \param LED  LED0 if setting LED0, LED1 if setting LED1, or other if setting both
//!
//! \return None
//
// *****************************************************************************
void HAL_LED_SetHigh(t_LED LED)
{
    if (LED == LED0)
    {
        GPIO_setOutputHighOnPin (GPIO_PORT_P1, BIT0);
    }
    else if (LED == LED1)
    {
        GPIO_setOutputHighOnPin (GPIO_PORT_P4, BIT6);
    }
    else
    {
        GPIO_setOutputHighOnPin (GPIO_PORT_P4, BIT6);
        GPIO_setOutputHighOnPin (GPIO_PORT_P1, BIT0);
    }
}

//*****************************************************************************
//
//! Set LED Low
//!
//! \param LED  LED0 if setting LED0, LED1 if setting LED1, or other if setting both
//!
//! \return None
//
// *****************************************************************************
void HAL_LED_SetLow(t_LED LED)
{
    if (LED == LED0)
    {
        GPIO_setOutputLowOnPin (GPIO_PORT_P1, BIT0);
    }
    else if (LED == LED1)
    {
        GPIO_setOutputLowOnPin (GPIO_PORT_P4, BIT6);
    }
    else
    {
        GPIO_setOutputLowOnPin (GPIO_PORT_P4, BIT6);
        GPIO_setOutputLowOnPin (GPIO_PORT_P1, BIT0);
    }
}

//*****************************************************************************
//
//! Initilize GPIOs for I2C/SMbus interface
//!
//! \return None
//
// *****************************************************************************
void HAL_I2C_InitGPIO(void)
{
    // Set I2C pins for SCL/SDA functionality
    GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P1, BIT6+ BIT7,
                                                GPIO_SECONDARY_MODULE_FUNCTION);
}

//*****************************************************************************
//
//! Get Base address used for SMBus/I2C interface
//!
//! \return I2C/SMBus Base address
//
// *****************************************************************************
uint32_t HAL_I2C_GetBaseAddr(void)
{
    return EUSCI_B0_BASE;
}


//*****************************************************************************
//
//! Initializes the TMP006 DRDY input 
//!
//! This input detects when TMP006 has a new conversion ready 
//!
//! \return None
//
// *****************************************************************************
void HAL_TMP006BOOST_DRDY_Init(void)
{
    //Enable P3.5 internal resistance as pull-Up resistance
    GPIO_setAsInputPinWithPullUpresistor(
            GPIO_PORT_P3,
            GPIO_PIN5
            );

    //P3.5 interrupt enabled
    GPIO_enableInterrupt(
            GPIO_PORT_P3,
            GPIO_PIN5
            );

    //P3.5 Hi/Lo edge
    GPIO_interruptEdgeSelect(
            GPIO_PORT_P3,
            GPIO_PIN5,
            GPIO_HIGH_TO_LOW_TRANSITION
            );


    //P3.5 IFG cleared
    GPIO_clearInterruptFlag(
            GPIO_PORT_P3,
            GPIO_PIN5
            );
    
}

//
// Interrupt Service Routines
//
//*****************************************************************************
//
//! I2C Interrupt Service Routine
//!
//! ISR for I2C module
//!   This ISR generates an event which should be declared in application
//!
//! \return None
//
// *****************************************************************************
#pragma vector=USCI_B0_VECTOR
__interrupt void HAL_I2C_ISR (void)
{
     // This event should be declared in Application
    if (HAL_I2C_Event() == true)
    {
        LPM3_EXIT;
    }
}

//*****************************************************************************
//
//! TMP006 DRDY Interrupt Service Routine
//!
//! ISR for DRDY Input
//!   This ISR generates an event which should be declared in application
//!
//! \return None
//
// *****************************************************************************
#pragma vector=PORT3_VECTOR
__interrupt void HAL_TMP006BOOST_Drdy_Isr(void)
{
    HAL_LED_SetHigh(LED0);  // Just to show when the packet is sent to PC
    
    // This event should be declared in Application
    if (HAL_TMP006BOOST_Drdy_Event() == true)
       LPM3_EXIT;

    //P3.5 IFG cleared
    GPIO_clearInterruptFlag(
            GPIO_PORT_P3,
            GPIO_PIN5
            );
    
}


//*****************************************************************************
//
//! Low level initialization 
//!
//! Disables the watchdog after reset
//!
//! \return 1: Initialize all variables
//
// *****************************************************************************
#ifdef __IAR_SYSTEMS_ICC__
int __low_level_init(void)
#elif defined (__TI_COMPILER_VERSION__)
int _system_pre_init(void)
#endif
{
    WDT_A_hold(WDT_A_BASE);             //Stop WDT
    return 1;
}


//*****************************************************************************
//
//! Dummy ISR 
//!
//! Dummy ISR in case an interrupt is enabled unintentionally
//!
//! \return none
//
// *****************************************************************************
#pragma vector=RTC_VECTOR,PORT2_VECTOR,TIMER0_A1_VECTOR,TIMER3_A1_VECTOR,     \
  USCI_A1_VECTOR,PORT1_VECTOR,TIMER1_A1_VECTOR,TIMER2_A1_VECTOR,   \
  TIMER0_B1_VECTOR,TIMER0_B0_VECTOR,            \
  SYSNMI_VECTOR,AES256_VECTOR, TIMER3_A0_VECTOR,     \
  TIMER1_A0_VECTOR, TIMER2_A0_VECTOR, COMP_E_VECTOR, UNMI_VECTOR
__interrupt void Dummy_ISR(void)
{
	while (1)
		;
}
