/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include "TI_SerialComm.h"
#include "Master_HAL.h"

//
// Type definitions
//
/*! Structure for SerialComm control*/
typedef struct
{
    volatile t_SerialComm_State eState; /*! Current State of SerialComm   */
    uint8_t RxLen;                      /*! Lenght of RX data (from PC)   */
    volatile uint8_t RxIndex;           /*! Current Index of RX data      */
    uint8_t *pRXBuf;                    /*! Pointer to RX data buffer     */
    uint8_t RX_MaxLen;                  /*! Maximum lenght of RX data     */
}t_SerialComm_struct;

//
//  Global variables
//
static Error_Function_t pError_Handler;     /*! Pointer to Error callback */
static t_SerialComm_struct s_SerialComm;    /*! Serial Comm structure     */

//
// Function definition
//
//*****************************************************************************
//
//! Initialize Serial Communication 
//! 
//! \param busClk  Clock frequency in Hz
//!
//! Initializes UART at 9600bps
//!
//! \return none
//
// *****************************************************************************
void TI_SerialComm_Init(uint32_t busClk)
{
    HAL_UART_Init(busClk);
    s_SerialComm.eState = SerialComm_Idle;
    pError_Handler = NULL;

}

//*****************************************************************************
//
//! Set Error callback 
//!
//! \param ErrorHandler  Pointer to callback function
//! 
//! The error callback will be called when an exception is found by the 
//! serial communication interface
//!
//! \return none
//
// *****************************************************************************
void TI_SerialComm_SetErrorHandler(Error_Function_t ErrorHandler)
{
    pError_Handler =  ErrorHandler;
}


//*****************************************************************************
//
//! Start packet reception  
//!
//! \param RXBuf  Pointer to reception buffer
//! \param RXLen  Maximum lenght of the reception buffer
//! 
//! Initializes variables to be ready to receive a new packet
//!
//! \return none
//
// *****************************************************************************
void TI_SerialComm_StartPacketReception(uint8_t *RXBuf, uint8_t RXLen)
{
    s_SerialComm.eState = SerialComm_WaitPacketHDR;
    s_SerialComm.RX_MaxLen = RXLen;
    s_SerialComm.pRXBuf = RXBuf;
    s_SerialComm.RxIndex = 0;
    s_SerialComm.RxLen = 0;
    
}

//*****************************************************************************
//
//! Get state of serial comm
//!
//! \return eState of Serial Comm (check t_SerialComm_struct)
//
// *****************************************************************************
t_SerialComm_State TI_SerialComm_GetState(void)
{
    return s_SerialComm.eState;
}

//*****************************************************************************
//
//! Checks if a packet has been received completely
//!
//! \return true if a packet was received, false if it hasn't
//
// *****************************************************************************
bool TI_SerialComm_PacketComplete(void)
{
    if (s_SerialComm.eState == SerialComm_CMDReceived)
    {
        return true;
    }
    else
    {
        return false;
    }
}

//*****************************************************************************
//
//! Get number of received bytes
//!
//! \return number of bytes received by Serial Comm
//
// *****************************************************************************
uint8_t TI_SerialComm_GetRxAvail(void)
{
    return s_SerialComm.RxIndex;
}

//*****************************************************************************
//
//! Transmit byte
//!
//! \param Data     Byte to be transmitted 
//!
//! Transmits a byte to Serial Comm interface, doesn't return until the TX 
//! buffer is empty
//! 
//! \return none
//
// *****************************************************************************
void TI_SerialComm_TxByte(uint8_t Data)
{
    HAL_UART_TxByte(Data);
}

//*****************************************************************************
//
//! Convert a nibble to hex value in ASCII
//!
//! \param nibble     expected to be 0x00-0x0F
//!
//! The nibble will be converted to its ASCII value (i.e. 0 = '0', 0xF = 'F')
//! 
//! \return Converted ASCII value of nibble
//
// *****************************************************************************
uint8_t nibble_to_hex(uint8_t nibble)
{
    uint8_t ret = nibble;
    if(nibble < 10)
    {
        ret += '0';
    }
    else
    {
        ret += 'A' - 10;
    }

    return(ret);
}


//*****************************************************************************
//
//! Convert a nibble to hex value in ASCII
//!
//! \param nibble     expected to be 0x00-0x0F
//!
//! The nibble will be converted to its ASCII value (i.e. 0 = '0', 0xF = 'F')
//! 
//! \return Converted ASCII value of nibble
//
// *****************************************************************************
int8_t char_to_hexint(uint8_t character)
{
    if((character >= '0') && (character <= '9'))
    {
        return (character - '0');
    }
    else if((character >= 'A') && (character <= 'F'))
    {
        return (character - 'A' + 10);
    }
    else if((character >= 'a') && (character <= 'f'))
    {
        return (character - 'a' + 10);
    }
    else
    {
        return(-1);
    }
}

//*****************************************************************************
//
//! Transmit an hexadecimal byte as ASCII 
//!
//! \param Data  Byte to be transmitted
//! 
//! Converts a byte to ASCII and sends it to Serial interface 
//!
//! \return none
//
// *****************************************************************************
void TI_SerialComm_TxHexByte(uint8_t Data)
{
    TI_SerialComm_TxByte(nibble_to_hex(Data >> 4));
    TI_SerialComm_TxByte(nibble_to_hex(Data & 0x0F));
}

//*****************************************************************************
//
//! Transmits a String 
//!
//! \param Data  Pointer to string to be transmitted (Null terminated)
//! 
//! Sends a string to the serial interface until it finds a Null char (0x00)
//!
//! \return none
//
// *****************************************************************************
void TI_SerialComm_TxString(uint8_t *Data)
{
    while(*Data != 0x00)
    {
        // Load data onto buffer
        TI_SerialComm_TxByte(*Data++);
    }
}

//*****************************************************************************
//
//! Transmits an Array 
//!
//! \param Data  Pointer to Array to be transmitted 
//! \param Len   Number of bytes to send from the array
//! 
//! Sends an array to the serial interface from Byte[0] to Byte[len-1]
//!
//! \return none
//
// *****************************************************************************
void TI_SerialComm_TxArray(uint8_t *Data, uint16_t len)
{
    uint16_t i;
    
    for (i=0; i < len; i++)
    {
        TI_SerialComm_TxByte(Data[i]);
    }
}

//*****************************************************************************
//
//! Transmits a response packet 
//!
//! \param response ResponseID indicates if the packet was sent correctly and\
//!                 the meaning of the packet. Values declared in Serial_Resp
//! \param ctrl  Value of SMBus control register
//! \param status Value of SMBus Status register
//! \param Txlen  Lenght of data sent via SMBus
//! \param TxAddr  Address of SMBus slave device
//! \param TxCmd  SMBus command sent to slave device
//! \param TxData  Data sent to the slave device via SMBus
//! \param RespData Response data received fromn the slave device via SMBus
//! \param RespLen  Lenght of response from SMBus slave
//! 
//! This function transmits a packet as expected by the PC application.
//! The format is:
//!  - Header (0x80)
//!  - Lenght (all fields from Ctrl to SMB RespData)
//!  - Control byte (SMBus ctrl)
//!  - Status byte (SMBus status)
//!  - TxLen (Number of bytes sent to slave)
//!  - TxAddr (Address of SMBus slave)
//!  - TxCmd (command sent to slave)
//!  - TxData (Data Bytes sent to slave)
//!  - Response ID (Check Serial_Resp)
//!  - Response data from slave
//!
//! \return none
//
// *****************************************************************************
void TI_SerialComm_TxPacket(uint8_t response, 
                            uint8_t ctrl, 
                            uint8_t status,
                            uint8_t Txlen,
                            uint8_t TxAddr, 
                            uint8_t TxCmd, 
                            uint8_t *TxData , 
                            uint8_t *RespData , 
                            uint8_t Resplen)
{
    TI_SerialComm_TxByte(SERIALCOMM_HEADER);    // Header in response
    TI_SerialComm_TxByte(4 + Txlen + Resplen);  // Lenght = ctrl+stat+txlen+TxData+Response+RespData
    TI_SerialComm_TxByte(ctrl);                 // Send ctrl
    TI_SerialComm_TxByte(status);               // Send status
    TI_SerialComm_TxByte(Txlen);                // Send TX lenght
    if (Txlen >= 2)
    {
        TI_SerialComm_TxByte(TxAddr);               // Send TX Addr
        TI_SerialComm_TxByte(TxCmd);                // Send TX lenght
        TI_SerialComm_TxArray(TxData, Txlen-2);       // Send TX data
    }
    TI_SerialComm_TxByte(response);             // Send response
    TI_SerialComm_TxArray(RespData, Resplen);   // Send Response data
}


//*****************************************************************************
//
//! Event called when new byte is received from Serial Comm
//!
//! \param Data  Received byte
//! 
//! \return true to wake-up CPU in ISR, false to stay in LPM
//
// *****************************************************************************
bool HAL_UART_DataRx_Event(uint8_t Data)
{
    uint8_t error_code = SERIALCOMM_NO_ERROR;
    bool ret = false; // If true, MCU will wake-up from LPM in ISR

    switch(s_SerialComm.eState)
    {
        case SerialComm_WaitPacketHDR:
          // Step1: check header
            if (Data == SERIALCOMM_HEADER)
            {
                s_SerialComm.eState = SerialComm_WaitLen;
            }
            else
            {
                s_SerialComm.eState = SerialComm_Error;
                error_code = SERIALCOMM_INVALID_HDR;
            }
        break;
        case SerialComm_WaitLen:
          // Step2: Check lenght
          if ((Data > s_SerialComm.RX_MaxLen) || (Data == 0))
          {
                // Expect data to be > 0 and less than max len
               s_SerialComm.eState = SerialComm_Error; 
               error_code = SERIALCOMM_INVALID_LEN;
          }
          else
          {
                s_SerialComm.RxLen = Data;  // Get the lenght 
                s_SerialComm.RxIndex = 0;   // Start index
                s_SerialComm.eState = SerialComm_RxData;
          }
        break;
        case SerialComm_RxData:
          // Step 3: Get the data
          if (s_SerialComm.RxIndex < s_SerialComm.RxLen)
          {
                // store the data 
                s_SerialComm.pRXBuf[s_SerialComm.RxIndex++] = Data;
                if (s_SerialComm.RxIndex == s_SerialComm.RxLen)
                {
                    // Step4: Wake-up CPU when full command is received
                    s_SerialComm.eState = SerialComm_CMDReceived;
                    ret = true; // Wake-up CPU
                }
          }
        break;
        default:
          ret = true; // Wake-up CPU
        break;
    }
        
    // Check for errors
    if (s_SerialComm.eState == SerialComm_Error)
    {
        if (pError_Handler != NULL)
        {
            // Call error handler
            pError_Handler(error_code);
            ret = true; // Wake-up CPU
        }
    }

    return ret;     
     
}



