/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdint.h>
#include "driverlib.h"
#include "Master_HAL.h"


//*****************************************************************************
//
//! Initialize UART and GPIOs used for RX/TX
//! 
//! \param busClk  Clock frequency in Hz
//!
//! Initializes UART at 9600bps
//!
//! \return none
//
// *****************************************************************************
void HAL_UART_Init(uint32_t busClk)
{
    /*

    * Select Port 2
    * Set Pin 0, 1 to input Secondary Module Function, (UCA0TXD/UCA0SIMO, UCA0RXD/UCA0SOMI).
    */
    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P2,
        GPIO_PIN0 + GPIO_PIN1,
        GPIO_SECONDARY_MODULE_FUNCTION
    );

    // Configure UART
#if (MCLK_MHZ==8)
    if ( STATUS_FAIL == EUSCI_A_UART_initAdvance(EUSCI_A0_BASE,
              EUSCI_A_UART_CLOCKSOURCE_SMCLK,
              52,
              1,
              0x49,
              EUSCI_A_UART_NO_PARITY,
              EUSCI_A_UART_LSB_FIRST,
              EUSCI_A_UART_ONE_STOP_BIT,
              EUSCI_A_UART_MODE,
              EUSCI_A_UART_OVERSAMPLING_BAUDRATE_GENERATION )){
         return;
     }
#else
#warning This option generates more code
    // Configure UART
    if ( STATUS_FAIL == EUSCI_A_UART_init(EUSCI_A0_BASE,
              EUSCI_A_UART_CLOCKSOURCE_SMCLK,
              CS_getSMCLK(CS_BASE),
              9600,
              EUSCI_A_UART_NO_PARITY,
              EUSCI_A_UART_LSB_FIRST,
              EUSCI_A_UART_ONE_STOP_BIT,
              EUSCI_A_UART_MODE,
              EUSCI_A_UART_LOW_FREQUENCY_BAUDRATE_GENERATION )){
         return;
     }
#endif
    

    EUSCI_A_UART_enable(EUSCI_A0_BASE);

    // Enable USCI_A0 RX interrupt
    EUSCI_A_UART_enableInterrupt(EUSCI_A0_BASE,
          EUSCI_A_UART_RECEIVE_INTERRUPT);  
   
}


//*****************************************************************************
//
//! Transmit byte to UART
//!
//! \param Data     Byte to be transmitted 
//!
//! Transmits a byte to UART, doesn't return until the TX buffer is empty
//! Note that this implementation uses polling for simplicity purposes.
//! 
//! \return None
//
// *****************************************************************************
void HAL_UART_TxByte(uint8_t Data)
{
    EUSCI_A_UART_transmitData(EUSCI_A0_BASE, Data);
    while (EUSCI_A_UART_getInterruptStatus(EUSCI_A0_BASE,EUSCI_A_UART_TRANSMIT_INTERRUPT_FLAG) == 0x00)
        ;

}

//
// Interrupt Service Routines
//
//*****************************************************************************
//
//! ISR for UART
//!
//! Interrupt Service Routine for UART serial communication with a PC
//! This ISR checks the format of the incoming packet from the PC, stores
//! the payload in a buffer, reports errors using a callback, and wakes-up
//! the CPU when necessary
//!
//! \return none
//
// *****************************************************************************
#pragma vector=USCI_A0_VECTOR
__interrupt void HAL_UART_ISR(void)
{
    uint8_t Data;
    
    switch(__even_in_range(UCA0IV,USCI_UART_UCTXCPTIFG))
    {
    case USCI_NONE: break;
    case USCI_UART_UCRXIFG:
        // Routine is executed when a byte is received via UART
        Data = EUSCI_A_UART_receiveData(EUSCI_A0_BASE);
        // Send the received byte to TI_Serial_Comm
        if (HAL_UART_DataRx_Event(Data) == true)
        {
            LPM3_EXIT;
        }
      break;
    case USCI_UART_UCTXIFG: break;
    case USCI_UART_UCSTTIFG: break;
    case USCI_UART_UCTXCPTIFG: break;
  }
}
