/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdint.h>
#include "msp430.h"
#include "Master_HAL.h"


//*****************************************************************************
//
//! Initialize UART and GPIOs used for RX/TX
//! 
//! \param busClk  Clock frequency in Hz
//!
//! Initializes UART at 9600bps
//!
//! \return none
//
// *****************************************************************************
void HAL_UART_Init(uint32_t busClk)
{
    /*
     * Select Port 1
     * Set Pin 1, 2 to input Secondary Module Function, (UCA0TXD/UCA0SIMO, UCA0RXD/UCA0SOMI).
     */
    P1SEL |= BIT1|BIT2;
    P1SEL2 |= BIT1|BIT2;

    UCA0CTL1 |= UCSWRST;                    // Put USCI in reset
    UCA0CTL1 |= UCSSEL_2;                   // Select SMCLK as source

    // Configure UART
#if (MCLK_MHZ==8)
    UCA0BR0 = 52;           // Set baudrate
    UCA0BR1 = 0x00;         //  ""
    UCA0MCTL = UCOS16| UCBRS_0 | UCBRF_1;   // Set Modulation
#else 
#error "Option not supported, add code in TI_SerialComm_HAL_G2553.c"
#endif
    

    UCA0CTL1 &= ~UCSWRST;                   // Start USCI 
    IE2 |= UCA0RXIE;                        // Enable USCI_A0 RX interrupt
 
}


//*****************************************************************************
//
//! Transmit byte to UART
//!
//! \param Data     Byte to be transmitted 
//!
//! Transmits a byte to UART, doesn't return until the TX buffer is empty
//! Note that this implementation uses polling for simplicity purposes.
//! 
//! \return None
//
// *****************************************************************************
void HAL_UART_TxByte(uint8_t Data)
{
  
    UCA0TXBUF = Data;               // Load data into USCI TX Buffer to send
    while(!(IFG2 & UCA0TXIFG))
    {
        ;                           // Wait for USCI Buffer to clear
    }
}

//
// Interrupt Service Routines
//
//*****************************************************************************
//
//! ISR for UART and I2C
//!
//! Interrupt Service Routine for UART serial communication with a PC
//! This ISR checks the format of the incoming packet from the PC, stores
//! the payload in a buffer, reports errors using a callback, and wakes-up
//! the CPU when necessary
//! G2xx3 also shares the ISR with I2C
//!
//! \return none
//
// *****************************************************************************
#pragma vector=USCIAB0TX_VECTOR, USCIAB0RX_VECTOR
__interrupt void HAL_UART_ISR(void)
{
    uint8_t Data;
    
    if(IFG2 & UCA0RXIFG)
    {
        // Routine is executed when a byte is received via UART
        Data = UCA0RXBUF;
        // Send the received byte to TI_Serial_Comm
        if (HAL_UART_DataRx_Event(Data) == true)
        {
            LPM3_EXIT;
        }
    }
    // If the event was not called by UART, check I2C
    else if (HAL_I2C_Event() == true)
    {
         LPM3_EXIT;
    }
}
