/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*
 * ======== TMP006.c ========
 * Driver controlling a SMBus TMP006 slave
 *
 *+----------------------------------------------------------------------------+
 * Please refer to the TI Design Guide for more details.
 * ---------------------------------------------------------------------------*/

#include <stdint.h>
#include <stdbool.h>
#include "Master_HAL.h"
#include "smbus.h"
#include "TMP006.h"

//*****************************************************************************
//
//! Initialize TMP006 
//!
//! Initializes TMP006 to default settings:
//!  1 sec cont conversions, DRDY disabled
//!
//! \return none
//
// *****************************************************************************
int8_t TMP006_Init( SMBus *SMB )
{
    uint16_t default_config = 0xF400;   // Reset TMP006

    return TMP006_WriteReg_wTimeout(SMB,                  // SMB struct
                                    TMP006_CONFIG_REG,     // Write Config register
                                    &default_config,        // Reset value
                                    100000);               // Timeout
}

//*****************************************************************************
//
//! Write a register of the TMP006 slave device
//!
//! \param SMB      Pointer to SMBus structure 
//! \param reg      Slave Register being written 
//! \param data     Pointer to data being written to slave (2 bytes)
//! \param timeout  Software timeout waiting for a response
//!
//! \return SMBUS_RET_ERROR if there was a SMBus interface error;\
//!         SMBUS_RET_OK if packet was sent OK
//
// *****************************************************************************
int8_t TMP006_WriteReg_wTimeout(SMBus *SMB, 
                                 uint8_t reg, 
                                 uint16_t *data, 
                                 uint32_t timeout)
{
    // TMP006 expects MSB first instead of LSB
    *data = __swap_bytes(*data);
    
    if (SMBus_masterWriteByteWord(SMB,                     // SMB struct
                                TMP006_SLAVE_ADDRESS,       // Slave Addr
                                reg,                        // SMB Command
                                (uint8_t *)data,           // DataPtr
                                2) != SMBUS_RET_OK)         // 2 byte data
    {
        return SMBUS_RET_ERROR;
    }
    
    if ((SMBus_masterWaitUntilDone (SMB, timeout) != SMBUS_RET_OK) ||
         (SMBus_getState(SMB) != SMBus_State_OK))
     {
        SMBus_masterReset(SMB); // Force a reset, so we can retry later
        return SMBUS_RET_ERROR;
     }
    
    return SMBUS_RET_OK;
}

//*****************************************************************************
//
//! Read a register from the TMP006 slave device
//!
//! \param SMB      Pointer to SMBus structure 
//! \param reg      Slave Register being read 
//! \param data     Pointer to data where value of register will be stored (2 bytes)
//! \param timeout  Software timeout waiting for a response
//!
//! \return SMBUS_RET_ERROR if there was a SMBus interface error;\
//!         SMBUS_RET_OK if packet was sent OK
//
// *****************************************************************************
int8_t TMP006_ReadReg_wTimeout(SMBus *SMB, 
                                 uint8_t reg, 
                                 uint16_t *data, 
                                 uint32_t timeout)
{
    if (SMBus_masterReadByteWord(SMB,                     // SMB struct
                                TMP006_SLAVE_ADDRESS,       // Slave Addr
                                reg,                        // SMB Command
                                (uint8_t *) data,                       // DataPtr
                                2) != SMBUS_RET_OK)         // 2 byte data
    {
         return SMBUS_RET_ERROR;
    }
    
    if ((SMBus_masterWaitUntilDone (SMB, timeout) != SMBUS_RET_OK) ||
         (SMBus_getState(SMB) != SMBus_State_OK))
     {
        SMBus_masterReset(SMB); // Force a reset, so we can retry later
        return SMBUS_RET_ERROR;
     }
    
    // Get the lenght of payload for response
    if (SMBus_getRxPayloadAvailable(SMB) != 2)
    {
        return SMBUS_RET_ERROR;             // RX Data size Error
    }
    
    
    // TMP006 expects MSB first instead of LSB
    *data = __swap_bytes(*data);
    
    return SMBUS_RET_OK;
}
