/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*
 * ======== main.c ========
 * SMBus Master - TI Design
 *
 * This example implements a SMBus Master device which can control a LED Dimmer 
 *  slave and a TMP006. 
 * The master communicates with a Host PC via UART, receiving commands and 
 *  sending responses from the slaves.
 *
 *                               /|\  /|\
 *                               Rp    Rp 
 *                  Master        |    |  
 *            -----------------   |    |  
 *           |             SDA*|<-|----+----> SMBus SDA 
 *           |                 |  |       
 *           |                 |  |       
 *           |             SCL*|<-+---------> SMBus SCL
 *   LED0 <--|Px.y*            |          
 *   LED1 <--|Pz.a*       Pb.c*|<--- TMP006 DRDY         
 *
 *                          * Check HAL for I/Os used for each device
 *+----------------------------------------------------------------------------+
 * Please refer to the TI Design Guide for more details.
 * ---------------------------------------------------------------------------*/

#include <stdint.h>
#include "TI_SerialComm.h"
#include "Master_HAL.h"
#include "smbus.h"
#include "TMP006.h"
#include "Dim430.h"

//
//  Global variables
//
static SMBus sSMBMaster;                             /*! SMBus Master Struct  */
static uint8_t RXSerial_Buff[SMB_MAX_PACKET_SIZE+4]; /*! Buffer for UART RX   */
static uint8_t Resp_Buff[SMB_MAX_PACKET_SIZE+4];     /*! Buffer for response \
                                                         from slave device    */
bool TMP006_Drdy_Event;     /*! Indicates if a TMP006 DRDY Event is detected */
static uint16_t Temp_word;  /*! Temporary variable */

enum Serial_Resp        /*! Responses to Serial Interface    */
{
    SC_OK =0,           /*! Command Executed OK, no response */
    SC_RX_ERROR,        /*! Command not executed, Error in RX packet from PC */ 
    SC_SMB_ERROR,       /*! Error in SMBus Interface  */
    SC_TMP006_V,        /*! Command OK, response to PC is TMP006 Voltage */
    SC_TMP006_T,        /*! Command OK, response to PC is TMP006 Temp */
    SC_TMP006_CONFIG,   /*! Command OK, response to PC is TMP006 Config */
    SC_TMP006_MANID,    /*! Command OK, response to PC is TMP006 Manuf ID */
    SC_TMP006_DEVID,    /*! Command OK, response to PC is TMP006 Device ID */
    SC_DIM430_DEVID,    /*! Command OK, response to PC is DIM430 Device ID */
    SC_DIM430_CONFIG,   /*! Command OK, response to PC is DIM430 Config */
    SC_DIM430_DUTY,     /*! Command OK, response to PC is DIM430 Duty cycle */
} Serial_Resp_e;



//
//  Function prototypes
//
extern void SMBus_MasterDemo(void);
static void Comm_ErrorHandler(uint8_t Error);


//*****************************************************************************
//
//! Main Function 
//!
//! SMBus Master demo.
//!  This demo initializes the slave devices TMP006 and DIM430 and waits for 
//!  2 events: DRDY and/or New commands from PC. 
//!  It acts as a bridge between PC and slaves.
//!
//! \return none
//
// *****************************************************************************
void main( void )
{
    uint8_t ret, RespLen; 
     
    // Set pointer to payload (ignoring the CMD which is in [0])
    uint8_t *SMBPayload = &RXSerial_Buff[1];   
    
    HAL_MCU_Init();             // Initialize default CLK and GPIO settings

    // Serial Comm (UART) is used to send/receive data from PC GUI
    TI_SerialComm_Init(MCLK_MHZ * 1000000);
    TI_SerialComm_SetErrorHandler(Comm_ErrorHandler);
    
    // Initialize GPIOs for I2C functionality
    HAL_I2C_InitGPIO();         
    
    // Initialize SMBus Master 
    SMBus_masterInit(&sSMBMaster, HAL_I2C_GetBaseAddr(), (MCLK_MHZ*1000000));
    // Enable SMBus Interrupts, after initializing I2C
    SMBus_masterEnableInt(&sSMBMaster);
    
    HAL_TMP006BOOST_DRDY_Init();    // Init interrupt for DRDY pin of TMP006
    TMP006_Drdy_Event= false;       // Clear flag to detect new TMP006 DRDY events

    __enable_interrupt();
    
    // Check if TMP006 is connected 
    if (TMP006_Init(&sSMBMaster) == SMBUS_RET_OK)
    {
        HAL_LED_SetHigh(LED0);
    }
    else
    {
        while (1)
        {   
            // Error initializing the TMP006
            HAL_LED_SetHigh(LED0);
            __delay_cycles(1000000);
            HAL_LED_SetLow(LED0);
            __delay_cycles(1000000);
        }
    }
    // Get ready for first packet
    TI_SerialComm_StartPacketReception(RXSerial_Buff, sizeof(RXSerial_Buff));
    
    while(1)
    {
        __disable_interrupt();
        // Sleep while a new packet is received from PC or the TMP006 has a new event
        while ((TI_SerialComm_PacketComplete() != true) && (TMP006_Drdy_Event == false))
        {
            // Go to sleep while we wait for a packet from PC
            _BIS_SR(LPM3_bits+GIE);
        }
        __enable_interrupt();
        
        ret = SC_RX_ERROR;      // return Parameter error by default
        RespLen = 0;            // Initialize variable 
        if (TI_SerialComm_PacketComplete() == true)
        {
            // Process the packet, the command from PC will be stored in 
            //  RXSerial_Buff[0]
            switch(RXSerial_Buff[0])
            {
                case 'C':
                  // Write Config register of TMP006
                  Temp_word = SMBPayload[0] | (SMBPayload[1] << 8);
                  if (TMP006_SET_CONFIG(&sSMBMaster, &Temp_word) != SMBUS_RET_OK)
                  {
                        ret = SC_SMB_ERROR;
                  }
                  else
                  {
                        ret = SC_OK;
                  }
                break;
                case 'c':
                  // Read Config register of TMP006
                  if (TMP006_READ_CONFIG(&sSMBMaster,  &Temp_word) != SMBUS_RET_OK)
                  {
                        ret = SC_SMB_ERROR;
                  }
                  else
                  {
                        Resp_Buff[0] = (uint8_t) Temp_word;   // Send MSB
                        Resp_Buff[1] = Temp_word>>8;          // Send LSB
                        ret = SC_TMP006_CONFIG;
                        RespLen = 2;
                  }
                break;
                case 'm':
                  // Read Manufacturer ID of TMP006
                  if (TMP006_READ_MANID(&sSMBMaster, &Temp_word) != SMBUS_RET_OK)
                  {
                        ret = SC_SMB_ERROR;
                  }
                  else
                  {
                        Resp_Buff[0] = (uint8_t) Temp_word;   // Send MSB
                        Resp_Buff[1] = Temp_word>>8;          // Send LSB
                        ret = SC_TMP006_MANID;
                        RespLen = 2;
                  }
                break;
                case 'd':
                  // Read Device ID of TMP006
                  if (TMP006_READ_DEVID(&sSMBMaster, &Temp_word) != SMBUS_RET_OK)
                  {
                        ret = SC_SMB_ERROR;
                  }
                  else
                  {
                        Resp_Buff[0] = (uint8_t) Temp_word;   // Send MSB
                        Resp_Buff[1] = Temp_word>>8;          // Send LSB
                        ret = SC_TMP006_DEVID;
                        RespLen = 2;
                  }
                break;
                case 'v':
                  // Read Sensor voltage of TMP006
                  if (TMP006_READ_SENS_V(&sSMBMaster, &Temp_word) != SMBUS_RET_OK)
                  {
                        ret = SC_SMB_ERROR;
                  }
                  else
                  {
                        Resp_Buff[0] = (uint8_t) Temp_word;   // Send MSB
                        Resp_Buff[1] = Temp_word>>8;          // Send LSB
                        ret = SC_TMP006_V;
                        RespLen = 2;
                  }
                break;
                case 't':
                  // Read Ambient temperature of TMP006
                  if (TMP006_READ_TEMP_AMB(&sSMBMaster, (uint16_t *) &Temp_word) != SMBUS_RET_OK)
                  {
                        ret = SC_SMB_ERROR;
                  }
                  else
                  {
                        Resp_Buff[0] = (uint8_t) Temp_word;   // Send MSB
                        Resp_Buff[1] = Temp_word>>8;          // Send LSB
                        ret = SC_TMP006_T;
                        RespLen = 2;
                  }
                break;
                case 'p':
                  // Read Device ID of LED Dimmer
                  if (DIM430_READ_DEVID(&sSMBMaster, (uint16_t *) &Resp_Buff) != SMBUS_RET_OK)
                  {
                        ret = SC_SMB_ERROR;
                  }
                  else
                  {
                        ret = SC_DIM430_DEVID;
                        RespLen = 2;
                  }
                break;
                case 'y':
                  // Read Dimmer configuration
                  if (DIM430_READ_CONFIG(&sSMBMaster, (uint16_t *) &Resp_Buff) != SMBUS_RET_OK)
                  {
                        ret = SC_SMB_ERROR;
                  }
                  else
                  {
                        ret = SC_DIM430_CONFIG;
                        RespLen = 2;
                  }
                break;
                case 'Y':
                  {
                      // Write Dimmer configuration
                      Temp_word = SMBPayload[0] | (SMBPayload[1] << 8);
                      if (DIM430_SET_CONFIG(&sSMBMaster, &Temp_word) != SMBUS_RET_OK)
                      {
                            ret = SC_SMB_ERROR;
                      }
                      else
                      {
                            ret = SC_OK;
                      }
                  }
                break;
                case 'u':
                  // Read Dimmer Duty cycle
                  if (DIM430_READ_DUTY(&sSMBMaster, (uint16_t *) &Resp_Buff) != SMBUS_RET_OK)
                  {
                        ret = SC_SMB_ERROR;
                  }
                  else
                  {
                        ret = SC_DIM430_DUTY;
                        RespLen = 2;
                  }
                break;
                case 'U':
                  {
                      // Write Dimmer Duty cycle
                      Temp_word = SMBPayload[0] | (SMBPayload[1] << 8);
                      if (DIM430_SET_DUTY(&sSMBMaster, &Temp_word) != SMBUS_RET_OK)
                      {
                            ret = SC_SMB_ERROR;
                      }
                      else
                      {
                            ret = SC_OK;
                      }
                  }
                break;
            }
            // Get ready for next packet
            TI_SerialComm_StartPacketReception(RXSerial_Buff, sizeof(RXSerial_Buff));

            // Send response to PC
            TI_SerialComm_TxPacket(ret, 
                                   sSMBMaster.ctrl.u8byte,     
                                   sSMBMaster.status.u8byte,   
                                   sSMBMaster.nwk.txLen+1,
                                   sSMBMaster.nwk.currentAddr,
                                   sSMBMaster.nwk.currentCmd,
                                   sSMBMaster.nwk.txBuffPtr,
                                   Resp_Buff, 
                                   RespLen);
        }
        
        // Read the new voltage and temp of TMP006
        if (TMP006_Drdy_Event == true )
        {
            // Read the new voltage of TMP006
            if (TMP006_READ_SENS_V(&sSMBMaster, &Temp_word) != SMBUS_RET_OK)
            {
                ret = SC_SMB_ERROR;
                RespLen = 0;  // don't send more data
            }
            else
            {
                Resp_Buff[0] = (uint8_t) Temp_word;   // Send MSB
                Resp_Buff[1] = Temp_word>>8;          // Send LSB
                ret = SC_TMP006_V;
                RespLen = 2;
                TMP006_Drdy_Event= false;       // Clear flag to detect new TMP006 DRDY events
            }
            // Send response to PC
            TI_SerialComm_TxPacket(ret, 
                                   sSMBMaster.ctrl.u8byte,     
                                   sSMBMaster.status.u8byte,   
                                   sSMBMaster.nwk.txLen+1,
                                   sSMBMaster.nwk.currentAddr,
                                   sSMBMaster.nwk.currentCmd,
                                   sSMBMaster.nwk.txBuffPtr,
                                   Resp_Buff, 
                                   RespLen);
            
            // Now Read Ambient temperature of TMP006
            if (TMP006_READ_TEMP_AMB(&sSMBMaster, (uint16_t *) &Temp_word) != SMBUS_RET_OK)
            {
                ret = SC_SMB_ERROR;
                RespLen = 0;  // don't send more data
            }
            else
            {
                Resp_Buff[0] = (uint8_t) Temp_word;   // Send MSB
                Resp_Buff[1] = Temp_word>>8;          // Send LSB
                ret = SC_TMP006_T;
                RespLen = 2;
                TMP006_Drdy_Event= false;       // Clear flag to detect new TMP006 DRDY events
            }
            // Send response to PC
            TI_SerialComm_TxPacket(ret, 
                                   sSMBMaster.ctrl.u8byte,     
                                   sSMBMaster.status.u8byte,   
                                   sSMBMaster.nwk.txLen+1,
                                   sSMBMaster.nwk.currentAddr,
                                   sSMBMaster.nwk.currentCmd,
                                   sSMBMaster.nwk.txBuffPtr,
                                   Resp_Buff, 
                                   RespLen);
            
            HAL_LED_SetLow(LED0); // Just to show that the packet was sent to PC

        }
    }

}

//*****************************************************************************
//
//! Serial Comm Error handler  
//! 
//! \param Error  Error code from SerialComm:
//!                 SERIALCOMM_INVALID_HDR: Invalid header
//!                 SERIALCOMM_INVALID_LEN: Invalid Lenght
//!
//! Callback for Serial Communication (UART) errors 
//!
//! \return none
//
// *****************************************************************************
static void Comm_ErrorHandler(uint8_t Error)
{
    // Error detected in Communication
    TI_SerialComm_TxByte(SC_RX_ERROR);
    // Restart communication
    TI_SerialComm_StartPacketReception(RXSerial_Buff, sizeof(RXSerial_Buff));
}


//*****************************************************************************
//
//! \brief   Event called when a new SMBus is detected 
//! 
//!         Events include Start, Stop, RX, TX, Errors
//
//! \return  true if device needs to wake-up, false if device stays in LPMx
//
//*****************************************************************************
bool HAL_I2C_Event(void)
{
    // Process SMBus ISR as Master
    // Note that the Master starts transfers in Main loop and this function 
    // handles the SMBus State machine. The main loop just waits for completion
     SMBus_masterProcessInt(&sSMBMaster);
     
     return false;
}
              
//*****************************************************************************
//
//! Timer Timeout Event
//!
//! Event called by HAL when Timeout timer is detected for devices without eUSCI
//!
//! \return  true to wake-up MCU, false to stay in LPMx
//
// *****************************************************************************
#ifdef __MSP430_HAS_USCI__
bool HAL_Timeout_Event(void)
{
    // Call the SMBUS function to handle a timeout error and restart the SMBUS
    SMBus_masterProcessTimeoutInt(&sSMBMaster);
    return(false);
}

#endif
                
//*****************************************************************************
//
//! \brief   Event called when TMP006 asserts DRDY line 
//! 
//! \return  true if device needs to wake-up, false if device stays in LPMx
//
//*****************************************************************************
bool HAL_TMP006BOOST_Drdy_Event(void)
{
    // Called when the TMP006 asserts the DRDY line
    TMP006_Drdy_Event = true;   // Just set a flag to get voltage and temp
    return true;                // Wake-up MCU
}
