/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*
 * ======== Slave_HAL_FR5969.c ========
 * Modules and GPIOs used by this HAL:
 *      =================================
 *      |   I2C         |   eUSCIB0     |
 *      |   I2C_SDA     |    P1.6       |
 *      |   I2C_SCL     |    P1.7       |
 *      |   Dimmer PWM  |    P1.4/TB0.1 |
 *      =================================
 * ---------------------------------------------------------------------------*/

#include <stdint.h>
#include <stdbool.h>
#include <stdlib.h>
#include "driverlib.h"
#include "Slave_HAL.h"

//
// Macros and Definitions
//
/*! PWM Period close to 200Hz using VLO as source clock */
#define PWM_PERIOD (9400/200) 

// 
// Function prototypes
//
static void MCU_Clock_Init(void);
static void MCU_GPIO_Init(void);

//*****************************************************************************
//
//! MCU initialization 
//!
//! Initializes the MSP430
//!  Sets GPIOs to default state
//!  Configures clocks
//!
//! \return None
//
// *****************************************************************************
void HAL_MCU_Init(void)
{
    MCU_GPIO_Init();    
    MCU_Clock_Init();

    // Turn off Temp sensor
    REFCTL0 |= REFTCOFF;
    REFCTL0 &= ~REFON;

    return;
}

//*****************************************************************************
//
//! Clock Initialization 
//!
//! This demo sets SMCLK/MCLK = DCO @ MCLK_MHZ (defined in project settings)
//!                ACLK = VLO 
//!
//! \return None
//
// *****************************************************************************
static void MCU_Clock_Init(void)
{
#if (MCLK_MHZ == 1)
    //Set DCO Frequency to 8MHz
    CS_setDCOFreq(CS_DCORSEL_0,CS_DCOFSEL_6);
    //Initialize SMCLK=DCOCLK and CLK DIVIDER set to 8 = 1Mhz
    CS_clockSignalInit(CS_SMCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_8);
    //Initialize MCLK=DCOCLK and CLK DIVIDER set to 8 = 1Mhz
    CS_clockSignalInit(CS_MCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_8);
#elif (MCLK_MHZ == 8)
    //Set DCO Frequency to 8MHz
    CS_setDCOFreq(CS_DCORSEL_0,CS_DCOFSEL_6);
    //Initialize SMCLK=DCOCLK and CLK DIVIDER set to 1 = 8Mhz
    CS_clockSignalInit(CS_SMCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
    //Initialize MCLK=DCOCLK and CLK DIVIDER set to 1 = 8Mhz
    CS_clockSignalInit(CS_MCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
#elif (MCLK_MHZ == 16)
    //Set DCO Frequency to 16MHz
    CS_setDCOFreq(CS_DCORSEL_1,CS_DCOFSEL_4);
    //Initialize SMCLK=DCOCLK and CLK DIVIDER set to 1 = 16Mhz
    CS_clockSignalInit(CS_SMCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
    //Initialize MCLK=DCOCLK and CLK DIVIDER set to 1 = 16Mhz
    CS_clockSignalInit(CS_MCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
#elif (MCLK_MHZ == 24)
    //Set DCO Frequency to 24MHz
    CS_setDCOFreq(CS_DCORSEL_1,CS_DCOFSEL_6);
    //Initialize SMCLK=DCOCLK and CLK DIVIDER set to 1 = 24Mhz
    CS_clockSignalInit(CS_SMCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
    //Initialize MCLK=DCOCLK and CLK DIVIDER set to 1 = 24Mhz
    CS_clockSignalInit(CS_MCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
#else
#error "Frequency not supported, change project setttings"
#endif

    //Initialize ACLK=VLO and no CLK DIVIDER set to 1
    CS_clockSignalInit(CS_ACLK,CS_VLOCLK_SELECT,CS_CLOCK_DIVIDER_1);

    // Crystals are not used
    CS_HFXTOff();
    CS_LFXTOff();

    return;

}


//*****************************************************************************
//
//! GPIO Initialization 
//!
//! Set GPIO pins to a default state
//!
//! \return None
//
// *****************************************************************************
static void MCU_GPIO_Init(void)
{
    // Set all pins as output low, 
    // except I2C (P1[6:7])
    // DRDY for TMP006 = P3.5 (not used for demo, but in order to stack boards up)
    // P4.2 which can be connected to LED0 from slave
    // P2.5/P2.6 are connected to Master RX/TX when stacked
    GPIO_setOutputLowOnPin(GPIO_PORT_P1, GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN2 |
                           GPIO_PIN3 | GPIO_PIN4 | GPIO_PIN5);
    GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN2 |
                           GPIO_PIN3 | GPIO_PIN4 | GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P3, GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN2 |
                           GPIO_PIN3 | GPIO_PIN4 | GPIO_PIN6 | GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN3 | 
                           GPIO_PIN4 | GPIO_PIN5 | GPIO_PIN6 | GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_PJ, GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN2 |
                           GPIO_PIN3 | GPIO_PIN4 | GPIO_PIN5 | GPIO_PIN6 |
                           GPIO_PIN7);

    GPIO_setAsOutputPin(GPIO_PORT_P1, GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN2 | GPIO_PIN3 |
                        GPIO_PIN4 | GPIO_PIN5);
    GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN2 | GPIO_PIN3 |
                        GPIO_PIN4 | GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P3, GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN2 | GPIO_PIN3 |
                        GPIO_PIN4 | GPIO_PIN6 | GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P4, GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN3 | GPIO_PIN4 | 
                        GPIO_PIN5 | GPIO_PIN6 | GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_PJ, GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN2 | GPIO_PIN3 |
                        GPIO_PIN4 | GPIO_PIN5 | GPIO_PIN6 | GPIO_PIN7);

    // Disable the GPIO power-on default high-impedance mode to activate
    // previously configured port settings
    PM5CTL0 &= ~LOCKLPM5;
    
    return;
}


//*****************************************************************************
//
//! Initialize Dimmer
//!
//! Initialize GPIO and Timer for LED Dimmer
//! FR5969 uses P1.4/TB0.1
//!
//! \return None
//
// *****************************************************************************
void HAL_Dimmer_Init(void)
{
    GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P1, BIT4,
                                                GPIO_PRIMARY_MODULE_FUNCTION);

    //Start timer, Set Frequency at ~200Hz (VLO/200), Timer disabled by default
    TIMER_B_configureUpMode(TIMER_B0_BASE,
                            TIMER_B_CLOCKSOURCE_ACLK,
                            TIMER_B_CLOCKSOURCE_DIVIDER_1,
                            PWM_PERIOD,
                            TIMER_B_TBIE_INTERRUPT_DISABLE,
                            TIMER_B_CCIE_CCR0_INTERRUPT_DISABLE,
                            TIMER_B_DO_CLEAR
                            );    
    
    //Initialize compare mode to generate PWM on TB0.1 using toggle-set mode
    TIMER_B_initCompare(TIMER_B0_BASE,
                        TIMER_B_CAPTURECOMPARE_REGISTER_1,
                        TIMER_B_CAPTURECOMPARE_INTERRUPT_DISABLE,
                        TIMER_B_OUTPUTMODE_TOGGLE_SET,
                        0   // PWM disabled by default
                        );
        
    return;
}

//*****************************************************************************
//
//! Enables the Dimmer Output
//!
//! \return None
//
// *****************************************************************************
void HAL_Dimmer_Enable(void)
{
    TIMER_B_startCounter(TIMER_B0_BASE,
                             TIMER_B_UP_MODE
                             );
    
    return;
}

//*****************************************************************************
//
//! Disables the Dimmer output
//!
//! \return None
//
// *****************************************************************************
void HAL_Dimmer_Disable(void)
{
    TIMER_B_stop(TIMER_B0_BASE);
    
    return;
}

//*****************************************************************************
//
//! Sets the Duty cycle percentage of the Dimmer PWM 
//!
//!  This function sets the duty cycle from 0% to 100% with 100 steps, but the 
//!   actual resolution of the PWM can be less than that depending on the 
//!   clock source and timer divider
//! 
//! \param Duty  Duty cycle from 0x0000=0% to 0x0064=100%
//!
//! \return Value written to duty cycle register
//
// *****************************************************************************
uint16_t HAL_Dimmer_SetDuty(uint16_t Duty)
{
    if (Duty > 100)
      Duty = 100;

    Timer_B_setCompareValue(TIMER_B0_BASE,
                            TIMER_B_CAPTURECOMPARE_REGISTER_1,
                            (Duty *PWM_PERIOD)/100);
    
    return Duty;
}

//*****************************************************************************
//
//! Initilize GPIOs for I2C/SMbus interface
//!
//! \return None
//
// *****************************************************************************
void HAL_I2C_InitGPIO(void)
{
    // Set I2C pins for SCL/SDA functionality
    GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P1, BIT6+ BIT7,
                                                GPIO_SECONDARY_MODULE_FUNCTION);
}

//*****************************************************************************
//
//! Get Base address used for SMBus/I2C interface
//!
//! \return I2C/SMBus Base address
//
// *****************************************************************************
uint32_t HAL_I2C_GetBaseAddr(void)
{
    return EUSCI_B0_BASE;
}

//
// Interrupt Service Routines
//
//*****************************************************************************
//
//! I2C Interrupt Service Routine
//!
//! ISR for I2C module
//!   This ISR generates an event which should be declared in application
//!
//! \return None
//
// *****************************************************************************
#pragma vector=USCI_B0_VECTOR
__interrupt void HAL_I2C_ISR (void)
{
    // This event should be declared in Application
    if (HAL_I2C_Event() == true)
    {
        LPM3_EXIT;
    }
}


//*****************************************************************************
//
//! Low level initialization 
//!
//! Disables the watchdog after reset
//!
//! \return 1: Initialize all variables
//
// *****************************************************************************
#ifdef __IAR_SYSTEMS_ICC__
int __low_level_init(void)
#elif defined (__TI_COMPILER_VERSION__)
int _system_pre_init(void)
#endif
{
    WDTCTL = WDTPW + WDTHOLD;   // Stop WDT
    return 1;
}


//*****************************************************************************
//
//! Dummy ISR 
//!
//! Dummy ISR in case an interrupt is enabled unintentionally
//!
//! \return none
//
// *****************************************************************************
#pragma vector=RTC_VECTOR,PORT2_VECTOR,TIMER0_A1_VECTOR,TIMER3_A1_VECTOR,     \
  USCI_A1_VECTOR,PORT1_VECTOR,TIMER1_A1_VECTOR,TIMER2_A1_VECTOR,   \
  TIMER0_B1_VECTOR,TIMER0_B0_VECTOR,            \
  SYSNMI_VECTOR,AES256_VECTOR, TIMER3_A0_VECTOR,     \
  TIMER1_A0_VECTOR, TIMER2_A0_VECTOR, COMP_E_VECTOR, UNMI_VECTOR
__interrupt void Dummy_ISR(void)
{
	while (1)
		;
}
