/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*
 * ======== main.c ========
 * SMBus Slave - TI Design
 *
 * This example implements a Slave Dimmer (DIM430) for TI Design
 *
 *                               /|\  /|\
 *                               Rp    Rp 
 *                  slave         |    |  
 *            -----------------   |    |  
 *           |             SDA*|<-|----+----> SMBus SDA
 *           |                 |  |       
 *           |                 |  |       
 *           |             SCL*|<-+---------> SMBus SCL
 * Dimmer <--|Px.y*            |          
 *
 *                          * Check HAL for I/Os used for each device
 *+----------------------------------------------------------------------------+
 * Please refer to the TI Design Guide for more details.
 * ---------------------------------------------------------------------------*/

#include <stdint.h>
#include "Slave_HAL.h"
#include "smbus.h"

//
// Macros and Definitions
//
/*! Configuration Virtual Register Address
 *   BIT15  BIT14   BIT13   BIT12   BIT11   BIT10   BIT9    BIT8
 *     -      -      -        -      -       -        -       -
 *   BIT7   BIT6    BIT5    BIT4    BIT3    BIT2    BIT1    BIT0
 *     -      -      -        -      -       -        -      EN0
 * EN0: Enables/Disables PWM output 
 */
#define DIM430_CONFIG_REG   0x00
/*! Duty Cycle Virtual Register Address
 *   BIT15  BIT14   BIT13   BIT12   BIT11   BIT10   BIT9    BIT8
 *   <---------------DUTY [15:8]-------------------------------->
 *   BIT7   BIT6    BIT5    BIT4    BIT3    BIT2    BIT1    BIT0
 *   <---------------DUTY [7:0]-------------------------------->
 * DUTY[15:0]: Sets duty cycle percentage from 0x0000 (0%) to 0x0064 (100%) 
 */
#define DIM430_DUTY_REG     0x01
/*! Device ID Virtual Register Address
 *   BIT15  BIT14   BIT13   BIT12   BIT11   BIT10   BIT9    BIT8
 *   <---------------ID [15:8]---------------------------------->
 *   BIT7   BIT6    BIT5    BIT4    BIT3    BIT2    BIT1    BIT0
 *   <---------------ID [7:0]----------------------------------->
 * ID[15:0]: Device ID , value fixed at 0x0043
 */
#define DIM430_DEV_ID_REG   0xFF

//
// Variables
//
static SMBus sSMBusSlave;                       /*! SMBus Structure           */
static uint8_t au8RxBuff[SMB_MAX_PACKET_SIZE];  /*! SMB Reception Buffer      */
static uint8_t au8TxBuff[SMB_MAX_PACKET_SIZE];  /*! SMB Transmission Buffer   */

static uint16_t u16Config = 0x00;               /*! Config Register           */
static const uint16_t u16DevID = 0x43;          /*! Device ID Register        */
static uint16_t u16Duty = 0x00;                 /*! PWM Duty cycle Register   */


//
//  Local Function prototypes
//
static void DIM430_WriteConfigReg(uint16_t data);
static void DIM430_WriteDuty(uint16_t data);

//*****************************************************************************
//
//! Main Function 
//!
//! SMBus Slave demo.
//!  This demo implements a simple LED Dimmer with three virtual registers:
//!  CONFIG_REGISTER: Enables/Disables PWM output
//!  DUTY_CYCLE_REGISTER: Sets the Duty cycle of Dimmer from 0% to 100%
//!  DEVICE_ID_REGISTER: Read-only register with Device ID
//!
//! \return none
//
// *****************************************************************************
void main( void )
{
     
    HAL_MCU_Init();             // Initialize default CLK and GPIO settings
    HAL_Dimmer_Init();          // Inialize GPIO and Timer for LED Dimmer

    HAL_I2C_InitGPIO();         // Initialize GPIOs for I2C functionality
    
    // Initialize SMBus Master 
    SMBus_slaveInit(&sSMBusSlave, HAL_I2C_GetBaseAddr());
    // Set slave address
    SMBus_slaveSetAddress(&sSMBusSlave, 0x43 );     
    
    // Set the RX and TX buffers for SMBus layers
    SMBus_slaveSetRxBuffer(&sSMBusSlave, au8RxBuff, sizeof(au8RxBuff));
    SMBus_slaveSetTxBuffer(&sSMBusSlave, au8TxBuff, sizeof(au8TxBuff));

    // Enable Interrupts
    SMBus_slaveEnableInt(&sSMBusSlave);
    
    while(1)
    {
    
         __disable_interrupt();
        // Sleep while a new packet is received from Master
        // In this example, the packet is handled in ISR since it's not 
        // CPU intensive
        {
            // Go to sleep while we wait for a packet from PC
            _BIS_SR(LPM3_bits + GIE);   
        }
        __enable_interrupt();
    }
}

//*****************************************************************************
//
//! \brief   Writes a new value to the Configuration Register
//
//! \param data Value being written to Config Register
//!             BIT0: EN0 enables/disables PWM output
//!             BIT15-BIT1: Ignored
//
//! \return  none
//
//*****************************************************************************
static void DIM430_WriteConfigReg(uint16_t data)
{
    u16Config = data & 0x01;    // Ignore all bits except BIT0
    
    // If set BIT0, enables PWM output, otherwise it disables it
    if (data & 0x01)
        HAL_Dimmer_Enable();
    else
        HAL_Dimmer_Disable();
}

//*****************************************************************************
//
//! \brief   Writes a value to the Duty Cycle register
//
//! \param data  Value written to Duty Cycle Register
//!             BIT15-BIT0: PWM duty cycle, 0x0000=0% 0x0064=100%
//
//! \return  none
//
//*****************************************************************************
static void DIM430_WriteDuty(uint16_t data)
{
    // The HAL will write the actual duty cycle depending on Source clocks
    u16Duty = HAL_Dimmer_SetDuty(data);
}


//*****************************************************************************
//
//! \brief   Event called when a new SMBus is detected 
//! 
//!         Events include Start, Stop, RX, TX, Errors
//
//! \return  true if device needs to wake-up, false if device stays in LPMx
//
//*****************************************************************************
bool HAL_I2C_Event(void)
{
    // Check the state of SMBus
    switch (SMBus_slaveProcessInt(&sSMBusSlave))
    {
        case SMBus_State_Slave_QCMD:
        break;
        case SMBus_State_Slave_CmdComplete:
            // Process the command
            switch (SMBus_slaveGetCommand(&sSMBusSlave))
            {
                case DIM430_CONFIG_REG:
                   if (SMBus_getRxPayloadAvailable(&sSMBusSlave) == 3)
                    {
                        // Write Config register 
                        DIM430_WriteConfigReg( sSMBusSlave.nwk.rxBuffPtr[1] + 
                                                (sSMBusSlave.nwk.rxBuffPtr[2]<<8));
                    }
                    // Send the contents of config register
                   sSMBusSlave.nwk.txBuffPtr[0] = u16Config & 0x00FF;
                   sSMBusSlave.nwk.txBuffPtr[1] = u16Config >> 8;
                   sSMBusSlave.nwk.txLen = 2;
                break;
                case DIM430_DUTY_REG:
                  if (SMBus_getRxPayloadAvailable(&sSMBusSlave) == 3)
                    {
                        // Write Config register 
                        DIM430_WriteDuty( sSMBusSlave.nwk.rxBuffPtr[1] + 
                                                (sSMBusSlave.nwk.rxBuffPtr[2]<<8));
                    }
                    // Send the contents of Duty register
                   sSMBusSlave.nwk.txBuffPtr[0] = u16Duty & 0x00FF;
                   sSMBusSlave.nwk.txBuffPtr[1] = u16Duty >> 8;
                   sSMBusSlave.nwk.txLen = 2;
                break;
                case DIM430_DEV_ID_REG:
                    // Ignore write commands, but just send ID 
                   sSMBusSlave.nwk.txBuffPtr[0] = u16DevID & 0x00FF;
                   sSMBusSlave.nwk.txBuffPtr[1] = u16DevID >> 8;
                   sSMBusSlave.nwk.txLen = 2;
                break;
            }
            return false;
        //break;
        case SMBus_State_Slave_FirstByte:
            // This demo doesn't pre-validate the command
        break;
       case SMBus_State_DataSizeError:
       case SMBus_State_PECError:
       case SMBus_State_Slave_Error:
       case SMBus_State_TimeOutError:
            // This demo simply updates the Status register when an error is 
            // detected. The Host can check this register
        break;
        default:
        break;
    }
    // Clear flags to be ready for next packet
    SMBus_processDone(&sSMBusSlave);
    
    return false;
}

//*****************************************************************************
//
//! \brief   Timeout event
//!
//!          Called when a SMBus timeout is detected
//!          Nothing is done in App Layer in this demo, but the interface gets
//!          a reset by the PHY layer
//
//! \return  true if device needs to wake-up, false if device stays in LPMx
//
//*****************************************************************************
#ifdef __MSP430_HAS_USCI__
bool HAL_Timeout_Event(void)
{
    SMBus_slaveProcessTimeoutInt(&sSMBusSlave);
    return false;
}
#endif
